// $Header: $
// Copyright (c) CODE Consulting and Development, s.r.o., Plzen. All rights reserved.
//#region Example
// This example shows how to subscribe to changes of multiple monitored items and display the value of the monitored item with 
// each change.

{
type
  TClientEventHandlers = class
    procedure OnDataChangeNotification(
      ASender: TObject;
      sender: OleVariant;
      const eventArgs: _EasyUADataChangeNotificationEventArgs);
  end;

procedure TClientEventHandlers.OnDataChangeNotification(
  ASender: TObject;
  sender: OleVariant;
  const eventArgs: _EasyUADataChangeNotificationEventArgs);
begin
  // Display the data
  // Remark: Production code would check eventArgs.Exception before accessing
  // eventArgs.AttributeData.
	WriteLn(eventArgs.Arguments.NodeDescriptor.ToString, ': ',
    eventArgs.AttributeData.ToString);
end;
}

class procedure SubscribeMultipleMonitoredItems.Main;
var
  Arguments: OleVariant;
  Client: TEasyUAClient;
  ClientEventHandlers: TClientEventHandlers;
  Handle: Cardinal;
  HandleArray: OleVariant;
  I: Cardinal;
  MonitoredItemArguments1, MonitoredItemArguments2, MonitoredItemArguments3, MonitoredItemArguments4:
    _EasyUAMonitoredItemArguments;
  MonitoringParameters: _UAMonitoringParameters;
  Url_String, NodeId_ExpandedText1, NodeId_ExpandedText2 : String;
  NodeId_ExpandedText3, NodeId_ExpandedText4: String;

begin

  Url_String := 'opc.tcp://MAINLINE:4980/Softing_dataFEED_OPC_Suite_Configuration1';
  // 4x OPC-Items: TEST/DNK/27.02.2019:
  NodeId_ExpandedText1 := 'nsu=Local Items;s=Local Items.EAK_Test2.EAK_Testwert1_I4';
  NodeId_ExpandedText2 := 'nsu=Local Items;s=Local Items.EAK_Test2.EAK_Testwert3_R4';
  NodeId_ExpandedText3 := 'nsu=Local Items;s=Local Items.EAK_Test2.EAK_Testwert5_String';
  NodeId_ExpandedText4 := 'nsu=Local Items;s=Local Items.EAK_Test2.EAK_Testwert7_Bool';

  // Instantiate the client object and hook events
  Client := TEasyUAClient.Create(nil);
  ClientEventHandlers := TClientEventHandlers.Create;
  Client.OnDataChangeNotification := ClientEventHandlers.OnDataChangeNotification;

  WriteLn('Subscribing...');
  MonitoringParameters := CoUAMonitoringParameters.Create;
  MonitoringParameters.SamplingInterval := 100; //1000;
  MonitoredItemArguments1 := CoEasyUAMonitoredItemArguments.Create;
  MonitoredItemArguments1.EndpointDescriptor.UrlString := Url_String;
  MonitoredItemArguments1.NodeDescriptor.NodeId.ExpandedText := NodeId_ExpandedText1;
  MonitoredItemArguments1.MonitoringParameters := MonitoringParameters;
  MonitoredItemArguments2 := CoEasyUAMonitoredItemArguments.Create;
  MonitoredItemArguments2.EndpointDescriptor.UrlString := Url_String;
  MonitoredItemArguments2.NodeDescriptor.NodeId.ExpandedText := NodeId_ExpandedText2;
  MonitoredItemArguments2.MonitoringParameters := MonitoringParameters;
  MonitoredItemArguments3 := CoEasyUAMonitoredItemArguments.Create;
  MonitoredItemArguments3.EndpointDescriptor.UrlString := Url_String;
  MonitoredItemArguments3.NodeDescriptor.NodeId.ExpandedText := NodeId_ExpandedText3;
  MonitoredItemArguments3.MonitoringParameters := MonitoringParameters;
  MonitoredItemArguments4 := CoEasyUAMonitoredItemArguments.Create;
  MonitoredItemArguments4.EndpointDescriptor.UrlString := Url_String;
  MonitoredItemArguments4.NodeDescriptor.NodeId.ExpandedText := NodeId_ExpandedText4;
  MonitoredItemArguments4.MonitoringParameters := MonitoringParameters;
  Arguments := VarArrayCreate([0, 3], varVariant);
  Arguments[0] := MonitoredItemArguments1;
  Arguments[1] := MonitoredItemArguments2;
  Arguments[2] := MonitoredItemArguments3;
  Arguments[3] := MonitoredItemArguments4;

  TVarData(HandleArray).VType := varArray or varVariant;
  TVarData(HandleArray).VArray := PVarArray(
    Client.SubscribeMultipleMonitoredItems(PSafeArray(TVarData(Arguments).VArray)));

  for I := VarArrayLowBound(HandleArray, 1) to VarArrayHighBound(HandleArray, 1) do
  begin
      Handle := Cardinal(HandleArray[I]);
      WriteLn('HandleArray[', I, ']: ', Handle);
  end;

  WriteLn('Processing monitored item changed events for 20 seconds...');
  PumpSleep(20*1000);

  WriteLn('Unsubscribing...');
  Client.UnsubscribeAllMonitoredItems;

  WriteLn('Waiting for 5 seconds...');
  Sleep(5*1000);

  WriteLn('Finished.');
  FreeAndNil(Client);
  FreeAndNil(ClientEventHandlers);
end;
//#endregion Example
